@@ -30,7 +30,6 @@ do_trace(ldns_resolver *local_res, ldns_rdf *name, ldns_rr_type t,
 	ldns_rr_list *new_nss_aaaa;
 	ldns_rr_list *final_answer;
 	ldns_rr_list *new_nss;
-	ldns_rr_list *hostnames;
 	ldns_rr_list *ns_addr;
 	uint16_t loop_count;
 	ldns_rdf *pop; 
@@ -195,9 +194,6 @@ do_trace(ldns_resolver *local_res, ldns_rdf *name, ldns_rr_type t,
 		return NULL;
 	}
 
-	hostnames = ldns_get_rr_list_name_by_addr(local_res, 
-			ldns_pkt_answerfrom(p), 0, 0);
-
 	new_nss = ldns_pkt_authority(p);
 	final_answer = ldns_pkt_answer(p);
 
@@ -220,6 +216,7 @@ do_trace(ldns_resolver *local_res, ldns_rdf *name, ldns_rr_type t,
  * the last argument prev_key_list, if not null, and type == DS, then the ds
  * rr list we have must all be a ds for the keys in this list
  */
+#ifdef HAVE_SSL
 ldns_status
 do_chase(ldns_resolver *res,
 	    ldns_rdf *name,
@@ -228,14 +225,13 @@ do_chase(ldns_resolver *res,
 	    ldns_rr_list *trusted_keys,
 	    ldns_pkt *pkt_o,
 	    uint16_t qflags,
-	    ldns_rr_list *prev_key_list,
+	    ldns_rr_list * ATTR_UNUSED(prev_key_list),
 	    int verbosity)
 {
 	ldns_rr_list *rrset = NULL;
 	ldns_status result;
 	ldns_rr *orig_rr = NULL;
 	
-	bool cname_followed = false;
 /*
 	ldns_rr_list *sigs;
 	ldns_rr *cur_sig;
@@ -289,7 +285,6 @@ do_chase(ldns_resolver *res,
 		/* answer might be a cname, chase that first, then chase
 		   cname target? (TODO) */
 		if (!rrset) {
-			cname_followed = true;
 			rrset = ldns_pkt_rr_list_by_name_and_type(pkt,
 					name,
 					LDNS_RR_TYPE_CNAME,
@@ -341,23 +336,13 @@ do_chase(ldns_resolver *res,
 
 /* if the answer had no answer section, we need to construct our own rr (for instance if
  * the rr qe asked for doesn't exist. This rr will be destroyed when the chain is freed */
- 	if (ldns_pkt_ancount(pkt) < 1) {
+	if (ldns_pkt_ancount(pkt) < 1) {
 		ldns_rr_set_type(orig_rr, type);
 		ldns_rr_set_owner(orig_rr, ldns_rdf_clone(name));
 	
 		chain = ldns_dnssec_build_data_chain(res, qflags, rrset, pkt, ldns_rr_clone(orig_rr));
 	} else {
-		/* chase the first answer? */
-		/*
-		printf("[XX] answer RR:\n");
-		ldns_rr_print(stdout, ldns_rr_list_rr(ldns_pkt_answer(pkt), 0));
-		chain = ldns_dnssec_build_data_chain(res,
-									  qflags,
-									  rrset,
-									  pkt,
-									  ldns_rr_list_rr(ldns_pkt_answer(pkt), 0)
-									  );
-		*/
+		/* chase the first answer */
 		chain = ldns_dnssec_build_data_chain(res, qflags, rrset, pkt, NULL);
 	}
 
@@ -405,42 +390,6 @@ do_chase(ldns_resolver *res,
 	/*	ldns_rr_free(orig_rr);*/
 
 	return result;
-#if 0
-	sigs = ldns_pkt_rr_list_by_name_and_type(pkt,
-			name,
-			LDNS_RR_TYPE_RRSIG,
-			LDNS_SECTION_ANY_NOQUESTION
-			);
-	/* these can contain sigs for other rrsets too! */
-	
-	if (rrset) {
-		printf("GOT RRSET:\n");
-		ldns_rr_list_print(stdout, rrset);
-		printf("\n");
-		printf("GOT SIGS:\n");
-		ldns_rr_list_print(stdout, sigs);
-		printf("\n");
-		
-	
-		for (sig_i = 0; sig_i < ldns_rr_list_rr_count(sigs); sig_i++) {
-			cur_sig = ldns_rr_clone(ldns_rr_list_rr(sigs, sig_i));
-			if (ldns_rdf2native_int16(ldns_rr_rrsig_typecovered(cur_sig)) == type) {
-			
-				keys = ldns_pkt_rr_list_by_name_and_type(pkt,
-						ldns_rr_rdf(cur_sig, 7),
-						LDNS_RR_TYPE_DNSKEY,
-						LDNS_SECTION_ANY_NOQUESTION
-						);
-			
-				result = ldns_verify_trusted(res, rrset, sigs, keys);
-			}
-			ldns_rr_free(cur_sig);
-		}
-		ldns_rr_list_deep_free(rrset);
-
-		printf("[chase] returning: %s\n", ldns_get_errorstr_by_id(result));
-		return result;
-	}
-#endif
 }
+#endif /* HAVE_SSL */
 
