@@ -38,7 +38,6 @@ retrieve_dnskeys(ldns_resolver *local_res, ldns_rdf *name, ldns_rr_type t,
 	ldns_pkt *p;
 	ldns_rr_list *new_nss_a;
 	ldns_rr_list *new_nss_aaaa;
-	ldns_rr_list *final_answer;
 	ldns_rr_list *new_nss;
 	ldns_rr_list *ns_addr;
 	ldns_rr_list *ns_addr2;
@@ -60,7 +59,6 @@ retrieve_dnskeys(ldns_resolver *local_res, ldns_rdf *name, ldns_rr_type t,
 	new_nss = NULL;
 	ns_addr = NULL;
 	ns_addr2 = NULL;
-	final_answer = NULL;
 	p = ldns_pkt_new();
 	res = ldns_resolver_new();
 	
@@ -665,7 +663,9 @@ main(int argc, char *argv[])
 
 	if (insecure) {
 		pkt = ldns_resolver_query(res, domain, LDNS_RR_TYPE_DNSKEY, LDNS_RR_CLASS_IN, LDNS_RD);
+		if (pkt) {
 		l = ldns_pkt_rr_list_by_type(pkt, LDNS_RR_TYPE_DNSKEY, LDNS_SECTION_ANY_NOQUESTION);
+		}
 	} else {
 		l = retrieve_dnskeys(res, domain, LDNS_RR_TYPE_DNSKEY, LDNS_RR_CLASS_IN, dns_root);
 	}
@@ -685,7 +685,8 @@ main(int argc, char *argv[])
 				
 				outputfile_buffer = ldns_buffer_new(300);
 				domain_str = ldns_rdf2str(ldns_rr_owner(k));
-				ldns_buffer_printf(outputfile_buffer, "K%s+%03u+%05u.key", domain_str, ldns_rdf2native_int8(ldns_rr_rdf(k, 2)), ldns_calc_keytag(k));
+				ldns_buffer_printf(outputfile_buffer, "K%s+%03u+%05u.key", domain_str, ldns_rdf2native_int8(ldns_rr_rdf(k, 2)),
+					(unsigned int) ldns_calc_keytag(k));
 				outputfile_str = ldns_buffer_export(outputfile_buffer);
 				
 				if (verbosity >= 1) {
