@@ -8,7 +8,9 @@
 #include "config.h"
 
 #include <ldns/ldns.h>
+#ifdef HAVE_SSL
 #include <openssl/ssl.h>
+#endif /* HAVE_SSL */
 
 #include <errno.h>
 
