@@ -13,7 +13,7 @@
 /**
  * \file
  *
- * Contains the definition of \ref ldns_pkt and its parts, as well
+ * Contains the definition of ldns_pkt and its parts, as well
  * as functions to manipulate those.
  */
 
@@ -37,6 +37,10 @@
 #include <ldns/rr.h>
 #include <sys/time.h>
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 /* opcodes for pkt's */
 enum ldns_enum_pkt_opcode {
 	LDNS_PACKET_QUERY = 0,
@@ -437,6 +441,20 @@ ldns_rr_list *ldns_pkt_authority(const ldns_pkt *p);
  * \return the section
  */
 ldns_rr_list *ldns_pkt_additional(const ldns_pkt *p);
+/**
+ * Return the packet's question, answer, authority and additional sections
+ * concatenated, in a new rr_list clone.
+ * \param[in] p the packet
+ * \return the rrs
+ */
+ldns_rr_list *ldns_pkt_all(const ldns_pkt *p);
+/**
+ * Return the packet's answer, authority and additional sections concatenated, 
+ * in a new rr_list clone.  Like ldns_pkt_all but without the questions.
+ * \param[in] p the packet
+ * \return the rrs except the question rrs
+ */
+ldns_rr_list *ldns_pkt_all_noquestion(const ldns_pkt *p);
 
 /**
  * return all the rr_list's in the packet. Clone the lists, instead
@@ -445,7 +463,7 @@ ldns_rr_list *ldns_pkt_additional(const ldns_pkt *p);
  * \param[in] s what section(s) to return
  * \return ldns_rr_list with the rr's or NULL if none were found
  */
-ldns_rr_list *ldns_pkt_get_section_clone(ldns_pkt *p, ldns_pkt_section s);
+ldns_rr_list *ldns_pkt_get_section_clone(const ldns_pkt *p, ldns_pkt_section s);
 
 /**
  * return all the rr with a specific name from a packet. Optionally
@@ -464,7 +482,7 @@ ldns_rr_list *ldns_pkt_rr_list_by_name(ldns_pkt *p, ldns_rdf *r, ldns_pkt_sectio
  * \param[in] s the packet's section
  * \return a list with the rr's or NULL if none were found
  */
-ldns_rr_list *ldns_pkt_rr_list_by_type(ldns_pkt *p, ldns_rr_type t, ldns_pkt_section s);
+ldns_rr_list *ldns_pkt_rr_list_by_type(const ldns_pkt *p, ldns_rr_type t, ldns_pkt_section s);
 /**
  * return all the rr with a specific type and type from a packet. Optionally
  * specify from which section in the packet
@@ -474,7 +492,7 @@ ldns_rr_list *ldns_pkt_rr_list_by_type(ldns_pkt *p, ldns_rr_type t, ldns_pkt_sec
  * \param[in] sec the packet's section
  * \return a list with the rr's or NULL if none were found
  */
-ldns_rr_list *ldns_pkt_rr_list_by_name_and_type(ldns_pkt *packet, ldns_rdf *ownername, ldns_rr_type type, ldns_pkt_section sec);
+ldns_rr_list *ldns_pkt_rr_list_by_name_and_type(const ldns_pkt *packet, const ldns_rdf *ownername, ldns_rr_type type, ldns_pkt_section sec);
 
 
 /**
@@ -608,7 +626,7 @@ void ldns_pkt_set_size(ldns_pkt *p, size_t s);
  * \param[in] p the packet
  * \param[in] timeval the timestamp
  */
-void ldns_pkt_set_timestamp(ldns_pkt *p, struct timeval);
+void ldns_pkt_set_timestamp(ldns_pkt *p, struct timeval timeval);
 /**
  * Set a packet's section count to x
  * \param[in] p the packet
@@ -826,8 +844,12 @@ bool ldns_pkt_safe_push_rr_list(ldns_pkt *pkt, ldns_pkt_section sec, ldns_rr_lis
 /**
  * check if a packet is empty
  * \param[in] p packet
- * \return true: empty, false: empty
+ * \return true: empty, false: not empty
  */
 bool ldns_pkt_empty(ldns_pkt *p);
 
+#ifdef __cplusplus
+}
+#endif
+
 #endif  /* LDNS_PACKET_H */
