@@ -14,7 +14,7 @@
  * \file
  *
  * Defines the  ldns_resolver structure, a stub resolver that can send queries and parse answers.
- * 
+ *
  */
 
 #ifndef LDNS_RESOLVER_H
@@ -28,26 +28,30 @@
 #include <ldns/packet.h>
 #include <sys/time.h>
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 /** Default location of the resolv.conf file */
 #define LDNS_RESOLV_CONF	"/etc/resolv.conf"
 /** Default location of the hosts file */
 #define LDNS_RESOLV_HOSTS	"/etc/hosts"
 
-#define LDNS_RESOLV_KEYWORD     -1	
+#define LDNS_RESOLV_KEYWORD     -1
 #define LDNS_RESOLV_DEFDOMAIN	0
 #define LDNS_RESOLV_NAMESERVER	1
 #define LDNS_RESOLV_SEARCH	2
 #define LDNS_RESOLV_SORTLIST	3
 #define LDNS_RESOLV_OPTIONS	4
-
-#define LDNS_RESOLV_KEYWORDS    5
+#define LDNS_RESOLV_ANCHOR	5
+#define LDNS_RESOLV_KEYWORDS    6
 
 #define LDNS_RESOLV_INETANY		0
 #define LDNS_RESOLV_INET		1
 #define LDNS_RESOLV_INET6		2
 
 #define LDNS_RESOLV_RTT_INF             0       /* infinity */
-#define LDNS_RESOLV_RTT_MIN             1       /* reacheable */
+#define LDNS_RESOLV_RTT_MIN             1       /* reachable */
 
 /**
  * DNS stub resolver structure
@@ -58,21 +62,21 @@ struct ldns_struct_resolver
 	uint16_t _port;
 
 	/** Array of nameservers to query (IP addresses or dnames) */
-	ldns_rdf **_nameservers; 
+	ldns_rdf **_nameservers;
 	/** Number of nameservers in \c _nameservers */
 	size_t _nameserver_count; /* how many do we have */
 
-        /**  Round trip time; 0 -> infinity. Unit: ms? */
-        size_t *_rtt;
+	/**  Round trip time; 0 -> infinity. Unit: ms? */
+	size_t *_rtt;
 
 	/**  Wether or not to be recursive */
 	bool _recursive;
 
 	/**  Print debug information */
 	bool _debug;
-	
+
 	/**  Default domain to add to non fully qualified domain names */
-	ldns_rdf *_domain; 
+	ldns_rdf *_domain;
 
 	/**  Searchlist array, add the names in this array if a query cannot be found */
 	ldns_rdf **_searchlist;
@@ -84,11 +88,15 @@ struct ldns_struct_resolver
 	uint8_t _retry;
 	/**  Time to wait before retrying */
 	uint8_t _retrans;
+	/**  Use new fallback mechanism (try EDNS, then do TCP) */
+	bool _fallback;
 
 	/**  Whether to do DNSSEC */
 	bool _dnssec;
 	/**  Whether to set the CD bit on DNSSEC requests */
 	bool _dnssec_cd;
+	/** Optional trust anchors for complete DNSSEC validation */
+	ldns_rr_list * _dnssec_anchors;
 	/**  Whether to use tcp or udp (tcp if the value is true)*/
 	bool _usevc;
 	/**  Whether to ignore the tc bit */
@@ -120,7 +128,7 @@ struct ldns_struct_resolver
 	uint16_t _axfr_i;
 	/* EDNS0 available buffer size */
 	uint16_t _edns_udp_size;
-	
+
 	/* Optional tsig key for signing queries,
 	outgoing messages are signed if and only if both are set
 	*/
@@ -139,7 +147,7 @@ typedef struct ldns_struct_resolver ldns_resolver;
 /**
  * Get the port the resolver should use
  * \param[in] r the resolver
- * \return the port number 
+ * \return the port number
  */
 uint16_t ldns_resolver_port(const ldns_resolver *r);
 
@@ -171,6 +179,13 @@ uint8_t ldns_resolver_retry(const ldns_resolver *r);
  */
 uint8_t ldns_resolver_retrans(const ldns_resolver *r);
 
+/**
+ * Get the truncation fallback status
+ * \param[in] r the resolver
+ * \return whether the truncation fallback mechanism is used
+ */
+bool ldns_resolver_fallback(const ldns_resolver *r);
+
 /**
  * Does the resolver use ip6 or ip4
  * \param[in] r the resolver
@@ -196,6 +211,18 @@ bool ldns_resolver_usevc(const ldns_resolver *r);
  * \return true: yes, fail, false: no, try the others
  */
 bool ldns_resolver_fail(const ldns_resolver *r);
+/**
+ * Does the resolver apply default domain name
+ * \param[in] r the resolver
+ * \return true: yes, false: no
+ */
+bool ldns_resolver_defnames(const ldns_resolver *r);
+/**
+ * Does the resolver apply search list
+ * \param[in] r the resolver
+ * \return true: yes, false: no
+ */
+bool ldns_resolver_dnsrch(const ldns_resolver *r);
 /**
  * Does the resolver do DNSSEC
  * \param[in] r the resolver
@@ -203,11 +230,17 @@ bool ldns_resolver_fail(const ldns_resolver *r);
  */
 bool ldns_resolver_dnssec(const ldns_resolver *r);
 /**
- * Does the resolver set the CD bit 
+ * Does the resolver set the CD bit
  * \param[in] r the resolver
  * \return true: yes, false: no
  */
 bool ldns_resolver_dnssec_cd(const ldns_resolver *r);
+/**
+ * Get the resolver's DNSSEC anchors
+ * \param[in] r the resolver
+ * \return an rr_list containg trusted DNSSEC anchors
+ */
+ldns_rr_list * ldns_resolver_dnssec_anchors(const ldns_resolver *r);
 /**
  * Does the resolver ignore the TC bit (truncated)
  * \param[in] r the resolver
@@ -330,14 +363,6 @@ void ldns_resolver_incr_nameserver_count(ldns_resolver *r);
  */
 void ldns_resolver_dec_nameserver_count(ldns_resolver *r);
 
-/**
- * Set the resolver's nameserver list directly. Note the
- * rr's should be A or AAAA 
- * \param[in] r the resolver
- * \param[in] ns the rr's to use as nameservers
- */
-void ldns_resolver_set_nameserver_rrlist(ldns_resolver *r, ldns_rr_list *ns);
-
 /**
  * Set the resolver's nameserver count directly.
  * \param[in] r the resolver
@@ -353,7 +378,7 @@ void ldns_resolver_set_nameserver_count(ldns_resolver *r, size_t c);
 void ldns_resolver_set_nameservers(ldns_resolver *r, ldns_rdf **rd);
 
 /**
- * Set the resolver's default domain. This gets appended when no 
+ * Set the resolver's default domain. This gets appended when no
  * absolute name is given
  * \param[in] r the resolver
  * \param[in] rd the name to append
@@ -408,6 +433,21 @@ void ldns_resolver_set_dnssec(ldns_resolver *r, bool b);
  * \param[in] b true: enable , false: don't use TCP
  */
 void ldns_resolver_set_dnssec_cd(ldns_resolver *r, bool b);
+/**
+ * Set the resolver's DNSSEC anchor list directly. RRs should be of type DS or DNSKEY.
+ * \param[in] r the resolver
+ * \param[in] l the list of RRs to use as trust anchors
+ */
+void ldns_resolver_set_dnssec_anchors(ldns_resolver *r, ldns_rr_list * l);
+
+/**
+ * Push a new trust anchor to the resolver. It must be a DS or DNSKEY rr
+ * \param[in] r the resolver.
+ * \param[in] rr the RR to add as a trust anchor.
+ * \return a status
+ */
+ldns_status ldns_resolver_push_dnssec_anchor(ldns_resolver *r, ldns_rr *rr);
+
 /**
  * Set the resolver retrans timeout (in seconds)
  * \param[in] r the resolver
@@ -415,6 +455,14 @@ void ldns_resolver_set_dnssec_cd(ldns_resolver *r, bool b);
  */
 void ldns_resolver_set_retrans(ldns_resolver *r, uint8_t re);
 
+/**
+ * Set whether the resolvers truncation fallback mechanism is used
+ * when ldns_resolver_query() is called.
+ * \param[in] r the resolver
+ * \param[in] fallback whether to use the fallback mechanism
+ */
+void ldns_resolver_set_fallback(ldns_resolver *r, bool fallback);
+
 /**
  * Set the resolver retry interval (in seconds)
  * \param[in] r the resolver
@@ -496,7 +544,7 @@ void ldns_resolver_set_nameserver_rtt(ldns_resolver *r, size_t pos, size_t value
 void ldns_resolver_set_random(ldns_resolver *r, bool b);
 
 /**
- * push a new nameserver to the resolver. It must be an IP
+ * Push a new nameserver to the resolver. It must be an IP
  * address v4 or v6.
  * \param[in] r the resolver
  * \param[in] n the ip address
@@ -505,16 +553,16 @@ void ldns_resolver_set_random(ldns_resolver *r, bool b);
 ldns_status ldns_resolver_push_nameserver(ldns_resolver *r, ldns_rdf *n);
 
 /**
- * push a new nameserver to the resolver. It must be an 
+ * Push a new nameserver to the resolver. It must be an
  * A or AAAA RR record type
  * \param[in] r the resolver
- * \param[in] rr the resource record 
+ * \param[in] rr the resource record
  * \return ldns_status a status
  */
 ldns_status ldns_resolver_push_nameserver_rr(ldns_resolver *r, ldns_rr *rr);
 
 /**
- * push a new nameserver rr_list to the resolver.
+ * Push a new nameserver rr_list to the resolver.
  * \param[in] r the resolver
  * \param[in] rrlist the rr_list to push
  * \return ldns_status a status
@@ -547,7 +595,7 @@ ldns_pkt* ldns_resolver_search(const ldns_resolver *r, const ldns_rdf *rdf, ldns
 ldns_status ldns_resolver_prepare_query_pkt(ldns_pkt **q, ldns_resolver *r, const  ldns_rdf *name, ldns_rr_type t, ldns_rr_class c, uint16_t f);
 
 /**
- * Send the query for name as-is 
+ * Send the query for name as-is
  * \param[out] **answer a pointer to a ldns_pkt pointer (initialized by this function)
  * \param[in] *r operate using this resolver
  * \param[in] *name query for this name
@@ -564,23 +612,23 @@ ldns_status ldns_resolver_send(ldns_pkt **answer, ldns_resolver *r, const ldns_r
  * \param[in] *r operate using this resolver
  * \param[in] *query_pkt query
  */
-ldns_status ldns_resolver_send_pkt(ldns_pkt **answer, const ldns_resolver *r, const ldns_pkt *query_pkt);
+ldns_status ldns_resolver_send_pkt(ldns_pkt **answer, ldns_resolver *r, ldns_pkt *query_pkt);
 
 /**
  * Send a query to a nameserver
  * \param[in] *r operate using this resolver
  * \param[in] *name query for this name
- * \param[in] *type query for this type (may be 0, defaults to A)
- * \param[in] *class query for this class (may be 0, default to IN)
+ * \param[in] *t query for this type (may be 0, defaults to A)
+ * \param[in] *c query for this class (may be 0, default to IN)
  * \param[in] flags the query flags
  * \return ldns_pkt* a packet with the reply from the nameserver
  * if _defnames is true the default domain will be added
  */
-ldns_pkt* ldns_resolver_query(const ldns_resolver *r, const ldns_rdf *name, ldns_rr_type type, ldns_rr_class class, uint16_t flags);
+ldns_pkt* ldns_resolver_query(const ldns_resolver *r, const ldns_rdf *name, ldns_rr_type t, ldns_rr_class c, uint16_t flags);
 
 
-/** 
- * Create a new resolver structure 
+/**
+ * Create a new resolver structure
  * \return ldns_resolver* pointer to new strcture
  */
 ldns_resolver* ldns_resolver_new(void);
@@ -605,44 +653,46 @@ ldns_status ldns_resolver_new_frm_fp(ldns_resolver **r, FILE *fp);
 ldns_status ldns_resolver_new_frm_fp_l(ldns_resolver **r, FILE *fp, int *line_nr);
 
 /**
- * configure a resolver by means of a resolv.conf file 
+ * Configure a resolver by means of a resolv.conf file
  * The file may be NULL in which case there will  be
  * looked the RESOLV_CONF (defaults to /etc/resolv.conf
  * \param[out] r the new resolver
  * \param[in] filename the filename to use
  * \return LDNS_STATUS_OK or the error
- */                             
+ */
 ldns_status ldns_resolver_new_frm_file(ldns_resolver **r, const char *filename);
 
-/**                             
+/**
  * Frees the allocated space for this resolver. Only frees the resolver pionter! You should probably be using _deep_free.
- * \param res resolver to free  
- */     
+ * \param res resolver to free
+ */
 void ldns_resolver_free(ldns_resolver *res);
 
-/**                             
+/**
  * Frees the allocated space for this resolver and all it's data
- * \param res resolver to free  
- */     
+ * \param res resolver to free
+ */
 void ldns_resolver_deep_free(ldns_resolver *res);
 
 /**
- *  get the next stream of RRs in a AXFR 
+ * Get the next stream of RRs in a AXFR
  * \param[in] resolver the resolver to use. First ldns_axfr_start() must be
  * called
  * \return ldns_rr the next RR from the AXFR stream
+ * After you get this returned RR (not NULL: on error), then check if 
+ * ldns_axfr_complete() is true to see if the zone transfer has completed.
  */
 ldns_rr* ldns_axfr_next(ldns_resolver *resolver);
 
 /**
- * returns true if the axfr transfer has completed (i.e. 2 SOA RRs and no errors were encountered 
+ * Returns true if the axfr transfer has completed (i.e. 2 SOA RRs and no errors were encountered
  * \param[in] resolver the resolver that is used
  * \return bool true if axfr transfer was completed without error
  */
 bool ldns_axfr_complete(const ldns_resolver *resolver);
 
 /**
- * returns a pointer to the last ldns_pkt that was sent by the server in the AXFR transfer
+ * Returns a pointer to the last ldns_pkt that was sent by the server in the AXFR transfer
  * uasable for instance to get the error code on failure
  * \param[in] res the resolver that was used in the axfr transfer
  * \return ldns_pkt the last packet sent
@@ -650,9 +700,22 @@ bool ldns_axfr_complete(const ldns_resolver *resolver);
 ldns_pkt *ldns_axfr_last_pkt(const ldns_resolver *res);
 
 /**
- * randomize the nameserver list in the resolver
+ * Randomize the nameserver list in the resolver
  * \param[in] r the resolver
  */
 void ldns_resolver_nameservers_randomize(ldns_resolver *r);
 
+/**
+ * Returns true if at least one of the provided keys is a trust anchor
+ * \param[in] r the current resolver
+ * \param[in] keys the keyset to check
+ * \param[out] trusted_keys the subset of trusted keys in the 'keys' rrset
+ * \return true if at least one of the provided keys is a configured trust anchor
+ */
+bool ldns_resolver_trusted_key(const ldns_resolver *r, ldns_rr_list * keys, ldns_rr_list * trusted_keys);
+
+#ifdef __cplusplus
+}
+#endif
+
 #endif  /* LDNS_RESOLVER_H */
