@@ -1736,6 +1736,9 @@ ldns_rr2canonical(ldns_rr *rr)
 	/*
 	 * lowercase the rdata dnames if the rr type is one
 	 * of the list in chapter 7 of RFC3597
+	 * Also added RRSIG, because a "Signer's Name" should be canonicalized
+	 * too. See dnssec-bis-updates-16. We can add it to this list because
+	 * the "Signer's Name"  is the only dname type rdata field in a RRSIG.
 	 */
 	switch(ldns_rr_get_type(rr)) {
         	case LDNS_RR_TYPE_NS:
@@ -1747,7 +1750,6 @@ ldns_rr2canonical(ldns_rr *rr)
         	case LDNS_RR_TYPE_MG:
         	case LDNS_RR_TYPE_MR:
         	case LDNS_RR_TYPE_PTR:
-        	case LDNS_RR_TYPE_HINFO:
         	case LDNS_RR_TYPE_MINFO:
         	case LDNS_RR_TYPE_MX:
         	case LDNS_RR_TYPE_RP:
@@ -1761,6 +1763,7 @@ ldns_rr2canonical(ldns_rr *rr)
         	case LDNS_RR_TYPE_SRV:
         	case LDNS_RR_TYPE_DNAME:
         	case LDNS_RR_TYPE_A6:
+        	case LDNS_RR_TYPE_RRSIG:
 			for (i = 0; i < ldns_rr_rd_count(rr); i++) {
 				ldns_dname2canonical(ldns_rr_rdf(rr, i));
 			}
@@ -2057,6 +2060,7 @@ static ldns_rr_descriptor rdata_field_descriptors[] = {
 {LDNS_RR_TYPE_NULL, "TYPE55", 1, 1, type_0_wireformat, LDNS_RDF_TYPE_NONE, LDNS_RR_NO_COMPRESS, 0 },
 {LDNS_RR_TYPE_NULL, "TYPE56", 1, 1, type_0_wireformat, LDNS_RDF_TYPE_NONE, LDNS_RR_NO_COMPRESS, 0 },
 {LDNS_RR_TYPE_NULL, "TYPE57", 1, 1, type_0_wireformat, LDNS_RDF_TYPE_NONE, LDNS_RR_NO_COMPRESS, 0 },
+	/* 58 */
 {LDNS_RR_TYPE_TALINK, "TALINK", 2, 2, type_talink_wireformat, LDNS_RDF_TYPE_NONE, LDNS_RR_NO_COMPRESS, 2 },
 {LDNS_RR_TYPE_NULL, "TYPE59", 1, 1, type_0_wireformat, LDNS_RDF_TYPE_NONE, LDNS_RR_NO_COMPRESS, 0 },
 {LDNS_RR_TYPE_NULL, "TYPE60", 1, 1, type_0_wireformat, LDNS_RDF_TYPE_NONE, LDNS_RR_NO_COMPRESS, 0 },
