@@ -259,17 +259,21 @@ ldns_str2rdf_int8(ldns_rdf **rd, const char *bytestr)
  */
 static int
 parse_escape(uint8_t *s, uint8_t *q) {
-	uint8_t val;
+	uint16_t val;
 	if (strlen((char *)s) > 3 &&
 	    isdigit((int) s[1]) &&
 	    isdigit((int) s[2]) &&
 	    isdigit((int) s[3])) {
 		/* cast this so it fits */
-		val = (uint8_t) ldns_hexdigit_to_int((char) s[1]) * 100 +
+		val = (uint16_t) ldns_hexdigit_to_int((char) s[1]) * 100 +
 		                ldns_hexdigit_to_int((char) s[2]) * 10 +
 		                ldns_hexdigit_to_int((char) s[3]);
-		*q = val;
-		return 3;
+		if (val > 255) {
+			/* outside range */
+			return 0;
+		}
+		*q = (uint8_t) val;
+                return 3;
 	} else {
 		s++;
 		if (*s == '\0' || isdigit((int) *s)) {
@@ -776,30 +780,30 @@ ldns_str2rdf_alg(ldns_rdf **rd, const char *str)
 }
 
 ldns_status
-ldns_str2rdf_unknown(ldns_rdf **rd, const char *str)
+ldns_str2rdf_unknown( ATTR_UNUSED(ldns_rdf **rd)
+		    , ATTR_UNUSED(const char *str)
+		    )
 {
 	/* this should be caught in an earlier time (general str2host for
 	   rr's */
-	rd = rd;
-	str = str;
 	return LDNS_STATUS_NOT_IMPL;
 }
 
 ldns_status
-ldns_str2rdf_tsig(ldns_rdf **rd, const char *str)
+ldns_str2rdf_tsig( ATTR_UNUSED(ldns_rdf **rd)
+		 , ATTR_UNUSED(const char *str)
+		 )
 {
-	/* there is no strign representation for TSIG rrs */
-	rd = rd;
-	str = str;
+	/* there is no string representation for TSIG rrs */
 	return LDNS_STATUS_NOT_IMPL;
 }
 
 ldns_status
-ldns_str2rdf_service(ldns_rdf **rd, const char *str)
+ldns_str2rdf_service( ATTR_UNUSED(ldns_rdf **rd)
+		    , ATTR_UNUSED(const char *str)
+		    )
 {
 	/* is this used? is this actually WKS? or SRV? */
-	rd = rd;
-	str = str;
 	return LDNS_STATUS_NOT_IMPL;
 }
 
