@@ -228,7 +228,8 @@ ldns_zone_glue_rr_list(const ldns_zone *z)
 			a = ldns_rr_list_rr(addr, j);
 			dname_a = ldns_rr_owner(a);
 
-			if (ldns_dname_is_subdomain(dname_a, ns_owner)) {
+			if (ldns_dname_is_subdomain(dname_a, ns_owner) ||
+				ldns_dname_compare(dname_a, ns_owner) == 0) {
 				/* GLUE! */
 				if (!ldns_rr_list_push_rr(glue, a)) goto memory_error;
 			}
@@ -344,8 +345,6 @@ ldns_zone_new_frm_fp_l(ldns_zone **z, FILE *fp, ldns_rdf *origin, uint32_t ttl,
 			/* a normal RR - as sofar the DNS is normal */
 			if (!ldns_zone_push_rr(newzone, rr)) goto error;
 
-			/*my_origin = ldns_rr_owner(rr);*/
-			my_ttl    = ldns_rr_ttl(rr); 	/* XXX: this seems like an error */
 		case LDNS_STATUS_SYNTAX_EMPTY:
 			/* empty line was seen */
 		case LDNS_STATUS_SYNTAX_TTL:
